#version 130

#define M_PI 3.14159265358979323846264338327950288

const float far = 5000.0, near = 5.0;
const float zs = -(far+near)/(far-near);
const float z0 = -2.0*far*near/(far-near);

const float d = 1.0;
const float FoV = 135.0;
const float halfFoV = 0.5 * (FoV * M_PI/180);
const float kFoV = (d + cos(halfFoV)) / ((d+1) * sin(halfFoV));
const float a = 9.0/16.0;

uniform mat4 modelToView;
uniform vec3 camera;

in vec3 model;
in vec3 normal_in;
in vec3 hue_in;

out vec3 normal;
out vec3 hue;
out vec3 fromCamera;

void main() {
	normal = normal_in;
	hue = hue_in;
	fromCamera = model - camera;

	vec4 q = modelToView * vec4(model, 1.0);
	float r = sqrt(q.x*q.x + q.y*q.y + q.z*q.z);

	vec2 p = normalize(q.xz);
	float ux = p.x, uz = p.y;
	float S = kFoV * (d + 1.0) / (d - uz);
	float depth = 2.0 * (-sign(q.z)*r-near) / (far-near) - 1.0;
	gl_Position = vec4(S * p.x, S/a * q.y/r, depth, 1.0);
}

// vim:syntax=c
